@@ -23,6 +23,7 @@ from __future__ import print_function
 
 import gzip
 import io
+import itertools
 import os
 import platform
 import pwd
@@ -222,6 +223,9 @@ class Install(install_misc.InstallBase):
         else:
             self.install_extras()
 
+        # Configure zsys
+        self.configure_zsys()
+
         self.next_region()
         self.db.progress('INFO', 'ubiquity/install/bootloader')
         self.copy_mok()
@@ -915,7 +919,7 @@ class Install(install_misc.InstallBase):
             arch, subarch = install_misc.archdetect()
 
             try:
-                if arch in ('amd64', 'i386'):
+                if arch in ('amd64', 'arm64', 'i386'):
                     from ubiquity.components import grubinstaller
                     while 1:
                         dbfilter = grubinstaller.GrubInstaller(None, self.db)
@@ -940,21 +944,6 @@ class Install(install_misc.InstallBase):
                                 self.db.set('grub-installer/bootdev', response)
                         else:
                             break
-                elif (arch in ('armel', 'armhf') and
-                      subarch in ('omap', 'omap4', 'mx5')):
-                    from ubiquity.components import flash_kernel
-                    dbfilter = flash_kernel.FlashKernel(None, self.db)
-                    ret = dbfilter.run_command(auto_process=True)
-                    if ret != 0:
-                        raise install_misc.InstallStepError(
-                            "FlashKernel failed with code %d" % ret)
-                elif arch == 'powerpc':
-                    from ubiquity.components import yabootinstaller
-                    dbfilter = yabootinstaller.YabootInstaller(None, self.db)
-                    ret = dbfilter.run_command(auto_process=True)
-                    if ret != 0:
-                        raise install_misc.InstallStepError(
-                            "YabootInstaller failed with code %d" % ret)
                 else:
                     raise install_misc.InstallStepError(
                         "No bootloader installer found")
@@ -965,6 +954,12 @@ class Install(install_misc.InstallBase):
             for bind in binds:
                 misc.execute('umount', '-f', self.target + bind)
 
+    def configure_zsys(self):
+        """ Configure zsys """
+        use_zfs = self.db.get('ubiquity/use_zfs')
+        if use_zfs:
+            misc.execute_root('/usr/share/ubiquity/zsys-setup', 'finalize')
+
     def copy_mok(self):
         if 'UBIQUITY_OEM_USER_CONFIG' in os.environ:
             return
@@ -1013,6 +1008,12 @@ class Install(install_misc.InstallBase):
         with cache.actiongroup():
             install_misc.get_remove_list(cache, to_remove, recursive)
 
+        with cache.actiongroup():
+            for cachedpkg in cache:
+                if cachedpkg.is_auto_removable and not cachedpkg.marked_delete:
+                    syslog.syslog("Autopurge %s" % cachedpkg.name)
+                    cachedpkg.mark_delete(auto_fix=False, purge=True)
+
         self.db.progress('SET', 1)
         self.progress_region(1, 5)
         fetchprogress = install_misc.DebconfAcquireProgress(
@@ -1168,8 +1169,26 @@ class Install(install_misc.InstallBase):
         if not found_cdrom:
             os.rename("%s.apt-setup" % sources_list, sources_list)
 
-        # this will not install non-free packages due to above divert
-        self.do_install(install_misc.query_recorded_installed())
+        # this will install free & non-free things, but not things
+        # that have multiarch Depends or Recommends. Instead, those
+        # will be installed by install_restricted_extras() later
+        # because this function runs before i386 foreign arch is
+        # enabled
+        cache = Cache()
+        filtered_extra_packages = install_misc.query_recorded_installed()
+        for package in filtered_extra_packages.copy():
+            pkg = cache.get(package)
+            if not pkg:
+                continue
+            candidate = pkg.candidate
+            dependencies = candidate.dependencies + candidate.recommends
+            all_deps = itertools.chain.from_iterable(dependencies)
+            for dep in all_deps:
+                if ':' in dep.name:
+                    filtered_extra_packages.remove(package)
+                    break
+
+        self.do_install(filtered_extra_packages)
 
         if found_cdrom:
             os.rename("%s.apt-setup" % sources_list, sources_list)
@@ -1402,8 +1421,10 @@ class Install(install_misc.InstallBase):
 
         arch, subarch = install_misc.archdetect()
 
-        if arch in ('amd64', 'i386'):
+        if arch in ('amd64', 'arm64', 'i386'):
             for pkg in ('grub', 'grub-efi', 'grub-efi-amd64',
+                        'grub-efi-arm64', 'grub-efi-arm64-signed',
+                        'flash-kernel', 'aarch64-laptops-support',
                         'grub-efi-amd64-signed', 'shim-signed', 'mokutil',
                         'lilo'):
                 if pkg not in keep:
@@ -1427,7 +1448,8 @@ class Install(install_misc.InstallBase):
             cache = self.restricted_cache
             for pkg in cache.keys():
                 if (cache[pkg].is_installed and
-                        cache[pkg].section.startswith('restricted/')):
+                        cache[pkg].candidate.section.startswith(
+                            'restricted/')):
                     difference.add(pkg)
             del cache
 
